//////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2011-12 Christos Christodoulopoulos and Tejaswini Deoskar

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 3 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package edinburgh.ccg;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;

import edinburgh.ccg.parse.ParseException;
import edinburgh.ccg.parse.Parser;
import edinburgh.ccg.util.FileUtils;
import edinburgh.ccg.util.Logger;
import edinburgh.ccg.util.Options;


/**
 * Main command-line interface.
 * 
 * @author Christos Christodoulopoulos
 */
public class CCG {

	public static void main(String[] args) {
		String usage = "CCG [options] parse_file\n" +
		"---options:\n" +
		"-m models_dir\tModels directory (for training/parsing)\n" +
		"-mt model_type\tType of model to use/train (base or lexcat)\n"+
		"-t [train_file]\t(Re)train the model using \"train_file\" or default\n" +
		"-s smooth_file\tSmoothed training using \"smooth_file\"\n" +
		"\t[-sf value]\tSmoothing interpolation factor\n" +
		"\t[-sc value]\tSmoothing cut-off frequency\n" +
		"-o [out_file]\tOutput file (default: parses.txt)\n" +
		"-b [alpha]\tUse beam search for pruning and specify the width\n" +
		"-c value\tRare category threshold during training\n" +
		"\t\t(Setting this value will cause the program to retrain)\n" +
		"-to mins\tTimeout for each parse (in minutes)\n" +
		"-pos\t\tUse PoS tags for unknown words\n" +
		"-ccg\t\tUse CCG-legal derivations only\n" +
		"-h\t\tThis message";

		if ((args.length > 0 && args[0].equals("-h")) || args.length == 0) {
			System.out.println("Usage:" + usage);
			System.exit(0);
		}
		Options o = new Options(args);
		//Print the options
		System.out.println(o.toString()+"######");
		FileUtils f = new FileUtils();
		Logger log = new Logger();

		Parser parser = new Parser(log);
		int sentenceNum = 0;

		//Normal (Viterbi) parsing
		if(o.getParseFile()!=null) { //if there is a file to parse
			try {
				String outFileName = o.getOutputFile();
				BufferedWriter parsesOut = f.createOut(outFileName);
				BufferedWriter depsOut = f.createOut(f.strip(outFileName)+".parg");
				parsesOut.write(o.toString()+"######\n");
				parsesOut.close();
				String line;
				BufferedReader fileReader = f.createIn(o.getParseFile());
				while ((line = fileReader.readLine()) != null){
					sentenceNum++;
					parsesOut = f.createOut(o.getOutputFile(), true);
					depsOut = f.createOut(f.strip(outFileName)+".parg", true);
					try {					
						System.out.print("Sent. "+sentenceNum+": ");
						line = line.trim(); 
						if(line.isEmpty()) {						
							throw (new ParseException("Empty input line"));
						}
						parser.parse(line);
						parser.printViterbiParse(sentenceNum, parsesOut, depsOut);
						System.out.println("...done!");
					} 
					catch (ParseException e) {
						System.out.println("...fail: "+e.toString());
						parsesOut.write("Sentence "+sentenceNum+": no parse: "+e.toString()+"\n#"+line);
						parsesOut.newLine();
						parsesOut.flush();
						depsOut.write("<s id=\""+sentenceNum+"\">: no parse: "+e.toString()+"\n");
						depsOut.write("<\\s>");
						depsOut.newLine();
					}
					parsesOut.close();
					depsOut.close();
				}
			}
			catch (IOException e){e.printStackTrace();}
		}
	}
}
