//////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2012 Christos Christodoulopoulos

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 3 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package edinburgh.ccg;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.text.DefaultCaret;

import edinburgh.ccg.parse.ParseException;
import edinburgh.ccg.parse.Parser;
import edinburgh.ccg.util.FileUtils;
import edinburgh.ccg.util.Logger;
import edinburgh.ccg.util.Options;


public class VizCCG extends JPanel implements ActionListener, PropertyChangeListener {

	private static final long serialVersionUID = 1L;

	private JPanel parsePanel;
	private JButton parseButton, parseFileButton, trainFileButton, 
	smoothFileButton, modelsDirButton, outFileButton;
	private JCheckBox retrainButton, beamButton, posButton, ccgButton;
	private JLabel parseFileLabel, smoothFileLabel, smoothCutoffLabel,
	smoothInterFactorLabel, beamLabel;
	private JRadioButton modelsBaseButton, modelsLexCatButton;
	private JTextField parseFileField, trainFileField, smoothFileField, 
	modelsDirField, beamField, smoothCutOffField, smoothInterpFactorField, outFileField;
	private JProgressBar progressBar;
	private JTextArea logTextArea = new JTextArea(10, 50);
	JScrollPane logPane;
	private ParseTask task;
	private JFileChooser fc, dirFC;

	private Options o = new Options();
	private FileUtils f = new FileUtils();
	private Logger log;

	public VizCCG(){
		super(new GridLayout(2, 1));
		
		fc = new JFileChooser(new File(".").getAbsolutePath());
		dirFC = new JFileChooser(new File(".").getAbsolutePath());
		dirFC.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		JTabbedPane tabbedPane = new JTabbedPane();
		makeParsePanel();
		parsePanel.setPreferredSize(new Dimension(660, 270));
		tabbedPane.addTab("Parsing", null, parsePanel, "");
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_P);
		add(tabbedPane);
		
		log = new Logger(logTextArea);
		logTextArea.setEditable(false);
		logTextArea.setLineWrap(true);
		logTextArea.setFont(UIManager.getFont("TabbedPane.font"));
		DefaultCaret caret = (DefaultCaret)logTextArea.getCaret();
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		logPane = new JScrollPane(logTextArea);
		logPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		add(logPane);
	}

	private void makeParsePanel(){
		parsePanel = new JPanel(false);
		
		//****** FILES ******
		modelsDirField = new JTextField();
		modelsDirField.setPreferredSize(new Dimension(100, 20));
		modelsDirField.setEditable(false);
		modelsDirField.setText(new File(o.getModelsDir()).getName());
		JLabel modelsDirLabel = new JLabel("Models directory:");
		modelsDirLabel.setToolTipText("Directory to store the prob. models");
		modelsDirLabel.setLabelFor(modelsDirField);
		modelsDirButton = new JButton("Browse...");
		modelsDirButton.addActionListener(this);
		trainFileField = new JTextField();
		trainFileField.setPreferredSize(new Dimension(100, 20));
		trainFileField.setEditable(false);
		JLabel trainFileLabel = new JLabel("Training file:");
		trainFileLabel.setLabelFor(trainFileField);
		trainFileButton = new JButton("Browse...");
		trainFileButton.addActionListener(this);
		smoothFileField = new JTextField();
		smoothFileField.setPreferredSize(new Dimension(100, 20));
		smoothFileField.setEditable(false);
		smoothFileLabel = new JLabel("Smoothing file:");
		smoothFileLabel.setToolTipText("File used for smoothing the counts of unseen words");
		smoothFileLabel.setLabelFor(smoothFileField);
		smoothFileButton = new JButton("Browse...");
		smoothFileButton.addActionListener(this);
		parseFileField = new JTextField();
		parseFileField.setPreferredSize(new Dimension(100, 20));
		parseFileField.setEditable(false);
		parseFileLabel = new JLabel("File to parse:");
		parseFileLabel.setLabelFor(parseFileField);
		parseFileButton = new JButton("Browse...");
		parseFileButton.addActionListener(this);
		outFileField = new JTextField();
		outFileField.setPreferredSize(new Dimension(100, 20));
		outFileField.setEditable(false);
		outFileField.setText(new File(o.getOutputFile()).getName());
		JLabel outFileLabel = new JLabel("Output file:");
		outFileLabel.setLabelFor(outFileField);
		outFileButton = new JButton("Browse...");
		outFileButton.addActionListener(this);
		
		JPanel filesPanel = new JPanel();
		filesPanel.setBorder(BorderFactory.createTitledBorder("Files"));
		filesPanel.setLayout(new GridLayout(5, 3));
		filesPanel.add(modelsDirLabel);
		filesPanel.add(modelsDirField);
		filesPanel.add(modelsDirButton);
		filesPanel.add(trainFileLabel);
		filesPanel.add(trainFileField);
		filesPanel.add(trainFileButton);
		filesPanel.add(smoothFileLabel);
		filesPanel.add(smoothFileField);
		filesPanel.add(smoothFileButton);
		filesPanel.add(parseFileLabel);
		filesPanel.add(parseFileField);
		filesPanel.add(parseFileButton);
		filesPanel.add(outFileLabel);
		filesPanel.add(outFileField);
		filesPanel.add(outFileButton);
		
		//****** PARAMETERS ******
		retrainButton = new JCheckBox("Retrain");
		retrainButton.setSelected(false);
		retrainButton.addActionListener(this);
		
		smoothCutOffField = new JTextField(Integer.toString(o.getSmoothCutOff()));
		smoothCutOffField.setPreferredSize(new Dimension(20, 20));
		smoothCutOffField.setEnabled(false);
		smoothCutoffLabel = new JLabel("Smoothing cut-off:");
		smoothCutoffLabel.setLabelFor(smoothCutOffField);
		smoothInterpFactorField = new JTextField(Double.toString(o.getSmoothLambda()));
		smoothInterpFactorField.setPreferredSize(new Dimension(40, 20));
		smoothInterpFactorField.setEnabled(false);
		smoothInterFactorLabel = new JLabel("Smoothing interp. factor:");
		smoothInterFactorLabel.setLabelFor(smoothInterpFactorField);

		beamButton = new JCheckBox("Parsing beam");
		beamButton.addActionListener(this);
		beamButton.setSelected(false);
		beamField = new JTextField(new BigDecimal(Double.toString(o.getAlpha())).toPlainString());
		beamField.setPreferredSize(new Dimension(55, 20));
		beamField.setEnabled(false);
		beamLabel = new JLabel("Beam width:");
		beamLabel.setLabelFor(beamField);
		
		posButton = new JCheckBox("Use PoS tags");
		posButton.addActionListener(this);
		posButton.setSelected(false);
		
		ccgButton = new JCheckBox("Use only valid CCG rules");
		ccgButton.addActionListener(this);
		ccgButton.setSelected(false);

		JPanel rightPanel = new JPanel();
		rightPanel.setBorder(BorderFactory.createTitledBorder("Parameters"));
		rightPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		rightPanel.add(retrainButton, c);
		c.gridx = 0;
		c.gridy = 1;
		rightPanel.add(posButton, c);
		c.gridx = 0;
		c.gridy = 2;
		rightPanel.add(ccgButton, c);
		c.gridx = 0;
		c.gridy = 3;
		rightPanel.add(beamButton, c);
		c.gridy = 4;
		rightPanel.add(beamLabel, c);
		c.gridx = 1;
		rightPanel.add(beamField, c);
		c.gridy = 5;
		c.gridx = 0;
		rightPanel.add(smoothCutoffLabel, c);
		c.gridx = 1;
		rightPanel.add(smoothCutOffField, c);
		c.gridy = 6;
		c.gridx = 0;
		rightPanel.add(smoothInterFactorLabel, c);
		c.gridx = 1;
		rightPanel.add(smoothInterpFactorField, c);
		
		//****** MODELS ******
		JPanel modelsPanel = new JPanel();
		modelsPanel.setBorder(BorderFactory.createTitledBorder("Prob. Model"));
		ButtonGroup modelsButtonGroup = new ButtonGroup();
		modelsBaseButton = new JRadioButton("Base");
		modelsLexCatButton = new JRadioButton("LexCat");
		modelsLexCatButton.setToolTipText("Extends the Base model with the lexical category of the sister node.");
		modelsButtonGroup.add(modelsBaseButton);
		modelsButtonGroup.add(modelsLexCatButton);
		modelsBaseButton.setSelected(true);
		modelsPanel.add(modelsBaseButton);
		modelsPanel.add(modelsLexCatButton);

		parseButton = new JButton("Parse");
		parseButton.setPreferredSize(new Dimension(100, 30));
		parseButton.addActionListener(this);

		progressBar = new JProgressBar(0, 100);
		progressBar.setValue(0);
		progressBar.setPreferredSize(new Dimension(500, 20));
		progressBar.setStringPainted(true);
		progressBar.setVisible(false);
		
		parsePanel.setLayout(new GridBagLayout());
		c.gridx = 0;
		c.gridy = 0;
		parsePanel.add(filesPanel, c);
		c.gridy = 1;
		parsePanel.add(modelsPanel, c);
		c.gridy = 0;
		c.gridx = 1;
		parsePanel.add(rightPanel, c);
		c.anchor = GridBagConstraints.CENTER;
		c.gridwidth = 2;
		c.gridy = 2;
		c.gridx = 0;
		parsePanel.add(parseButton, c);
		c.gridy = 3;
		c.insets = new Insets(10, 0, 0, 10);
		parsePanel.add(progressBar, c);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == parseFileButton){
			int returnVal = fc.showOpenDialog(parsePanel);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				parseFileField.setText(file.getName());
				o.setParseFile(file.getPath());
			}
		}
		else if (e.getSource() == modelsDirButton){
			int returnVal = dirFC.showOpenDialog(parsePanel);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File dir = dirFC.getSelectedFile();
				modelsDirField.setText(dir.getName());
				o.setModelsDir(dir.getPath());
			}
		}
		else if (e.getSource() == trainFileButton){
			int returnVal = fc.showOpenDialog(parsePanel);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				trainFileField.setText(file.getName());
				o.setTrainFile(file.getPath());
			}
		}
		else if (e.getSource() == smoothFileButton){
			int returnVal = fc.showOpenDialog(parsePanel);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				smoothFileField.setText(file.getName());
				o.setSmoothFile(file.getPath());
				smoothCutOffField.setEnabled(true);
				smoothInterpFactorField.setEnabled(true);
			}
		}
		else if (e.getSource() == outFileButton){
			int returnVal = fc.showOpenDialog(parsePanel);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				outFileField.setText(file.getName());
				o.setOutputFile(file.getPath());
			}
		}
		else if (e.getSource() == posButton){
			boolean selected = posButton.isSelected();
			o.setUseTags(selected);
			o.setRetrain(selected);
			retrainButton.setSelected(selected);
			setState(!selected, smoothFileButton, smoothFileField, smoothFileLabel);
		}
		else if (e.getSource() == beamButton){
			boolean selected = beamButton.isSelected();
			beamField.setEnabled(selected);
			o.setBeamSearch(selected);
		}
		else if (e.getSource() == retrainButton){
			o.setRetrain(retrainButton.isSelected());
		}
		else if (e.getSource() == parseButton){
			progressBar.setValue(0);
			if (modelsBaseButton.isSelected()) o.setModelType("base");
			else if (modelsLexCatButton.isSelected()) o.setModelType("lexcat");
			if (o.getParseFile() == null){
				String msg;
				msg = "No file to parse!";
				JOptionPane.showMessageDialog(this, msg, "Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (o.isTrain() && o.getTrainFile() == null){
				JOptionPane.showMessageDialog(this, "No training file!","Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (o.isBeamSearch()){
				if (beamField.getText() != "") o.setAlpha(Double.parseDouble(beamField.getText()));
				else {
					JOptionPane.showMessageDialog(this, "Please set a beam width!",
							"Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
			if (o.isSmoothTrain()){
				if (smoothCutOffField.getText() != "") o.setSmoothCutOff(Integer.parseInt(smoothCutOffField.getText()));
				else {
					JOptionPane.showMessageDialog(this, "Please set smooth cut-off threshold!",
							"Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
				if (smoothInterpFactorField.getText() != "") 
					o.setSmoothInterpFactor(Double.parseDouble(smoothInterpFactorField.getText()));
				else {
					JOptionPane.showMessageDialog(this, "Please set smooth interpolation threshold!",
							"Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
			logTextArea.setText("");
			parseButton.setEnabled(false);
			progressBar.setVisible(true);
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			task = new ParseTask();
			task.addPropertyChangeListener(this);
			task.execute();
		}
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals("progress")) {
			int progress = (Integer) evt.getNewValue();
			progressBar.setValue(progress);
		} 
	}

	private static void createAndShowGUI() {
		JFrame frame = new JFrame("StatOpenCCG Parser");
		frame.setIconImage(new ImageIcon("lib/ccg.png").getImage());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		//Set location TODO: relative to screen
		frame.setLocation(100, 100);
		frame.setResizable(false);
		frame.add(new VizCCG(), BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);
	}
	
	private void setState(boolean state, JComponent...components){
		for (JComponent comp:components){
			comp.setEnabled(state);
		}
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
				catch (ClassNotFoundException e) {e.printStackTrace();} 
				catch (InstantiationException e) {e.printStackTrace();} 
				catch (IllegalAccessException e) {e.printStackTrace();} 
				catch (UnsupportedLookAndFeelException e) {e.printStackTrace();}
				createAndShowGUI();
			}
		});
	}

	class ParseTask extends SwingWorker<Void, Void> {
		@Override
		public Void doInBackground() {
			try {
				String outFileName = o.getOutputFile();
				BufferedWriter parsesOut = f.createOut(outFileName);
				BufferedWriter depsOut = f.createOut(f.strip(outFileName)+".parg");
				parsesOut.write(o.toString()+"######\n");
				parsesOut.close();
				String line;
				int totalSents = 0;
				BufferedReader fileReader = f.createIn(o.getParseFile());
				while ((line = fileReader.readLine()) != null && line!=""){
					totalSents++;
				}
				fileReader.close();

				int sentNo = 0;
				//Initialize progress bar.
				setProgress(0);
				fileReader = f.createIn(o.getParseFile());
				Parser parser = new Parser(log);
				//Parse each sentence
				while ((line = fileReader.readLine()) != null && line!=""){
					sentNo++;
					parsesOut = f.createOut(o.getOutputFile(), true);
					depsOut = f.createOut(f.strip(outFileName)+".parg", true);
					try {				
						log.print("Sent. "+sentNo+":");
						parser.parse(line);
						parser.printViterbiParse(sentNo, parsesOut, depsOut);
						log.println("...done!");
					}
					catch (ParseException e) {
						log.println("...fail: "+e.toString());
						parsesOut.write("Sentence "+sentNo+": no parse: "+e.toString()+"\n#"+line);
						parsesOut.newLine();
						parsesOut.flush();
						depsOut.write("<s id=\""+sentNo+"\">: no parse: "+e.toString()+"\n");
						depsOut.write("<\\s>");
						depsOut.newLine();
					}
					parsesOut.close();
					depsOut.close();
					int prog = Math.round(sentNo*100/totalSents);
					setProgress(prog);
				}
			}
			catch (IOException e){e.printStackTrace();}
			return null;
		}

		@Override
		public void done() {
			parseButton.setEnabled(true);
			setCursor(null);
		}
	}
}
