//////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2008 Christos Christodoulopoulos

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package edinburgh.ccg.ccgCheck;

import edinburgh.ccg.ccgCheck.utils.ArgStack;
import edinburgh.ccg.ccgCheck.utils.AtomCat;
import edinburgh.ccg.ccgCheck.utils.Category;
import edinburgh.ccg.ccgCheck.utils.ComplexCat;
import edinburgh.ccg.ccgCheck.utils.Slash;
import edinburgh.ccg.ccgCheck.utils.TargetCat;

public class CategoryCreator {
		
	public Category createCat(String flatCatStr){
		Category cat;
		if (isComplex(flatCatStr)) cat = createComplexCat(flatCatStr);
		else cat = createAtomCat(flatCatStr);
		return cat;
	}

	private Category createAtomCat(String catText){
		return new AtomCat(catText);
	}
	
	private Category createComplexCat(String catText){
		Category target = null;
		ArgStack args = null;
		String targetPart, argsPart;
		int mainSlashInd;
		// Remove the outer brackets (if any)
		catText = removeOuterBrackets(catText);
		// Find the main slash
		mainSlashInd = getMainSlash(catText);
		// Get the arguments part: right of the main slash
		argsPart = catText.substring(mainSlashInd+1);
		// Create the argument stack
		args = createArgStack(argsPart, catText.charAt(mainSlashInd));
		// Get the target part
		targetPart = catText.substring(0, mainSlashInd);
		// Create the target
		target = createTarget(targetPart, target, args);
		// Create the final category
		return new ComplexCat((TargetCat)target,args);
	}
	
	private ArgStack createArgStack(String argsText, char slash){
		Category argCat;
		// Remove the outer brackets (if any)
		argsText = removeOuterBrackets(argsText);
		// If arguments are complex
		if (isComplex(argsText))
			argCat = createComplexCat(argsText);
		else
			argCat = createAtomCat(argsText);
		// Create the argument stack
		return new ArgStack(createSlash(slash), argCat);
	}
	
	private Category createTarget(String targetText, Category target, ArgStack args){
		String targetPart, argsPart;
		int mainSlashInd;
		// Remove the outer brackets (if any)
		targetText = removeOuterBrackets(targetText);
		// If target is complex
		if (isComplex(targetText)){
			// Find the main slash
			mainSlashInd = getMainSlash(targetText);
			// Get the arguments part: right of the main slash
			argsPart = targetText.substring(mainSlashInd+1);
			// Create the argument stack, but now add it at the front of the existing stack
			args.addFront(createArgStack(argsPart, targetText.charAt(mainSlashInd)));
			// Get the target part
			targetPart = targetText.substring(0, mainSlashInd);
			// Create the target
			target = createTarget(targetPart, target, args);
		}
		else
			target = createAtomCat(targetText);
		return target;
	}
	
	private boolean isComplex(String catText){
		if (catText.indexOf('/')>0 || catText.indexOf('\\')>0)
			return true;
		else
			return false;
	}
	
	private String removeOuterBrackets(String text){
		int bracketCnt = 0;
		int charCnt=1;
		if (text.charAt(0)=='('){
			bracketCnt++;
			for (charCnt=1; charCnt<text.length(); charCnt++){
				if (text.charAt(charCnt)=='(') bracketCnt++;
				else if (text.charAt(charCnt)==')'){
					if (bracketCnt==1){
						break;
					}
					else bracketCnt--;
				}
			}
			// This means that there is a set of outer brackets
			if (charCnt+1==text.length()){
				return new String(text.substring(1,text.length()-1));
			}
		}
		return text;
	}
	
	public int getMainSlash(String catText){
		int bracketCnt = 0;
		int searchInd = 0;
		// Get the first slash after the brackets
		if (catText.charAt(0)=='('){
			for (searchInd=0; searchInd<catText.length(); searchInd++){
				if (catText.charAt(searchInd)=='(') bracketCnt++;
				else if (catText.charAt(searchInd)==')'){
					if (bracketCnt==1){
						break;
					}
					else bracketCnt--;
				}
			}
		}
		if (catText.indexOf('/',searchInd)>0){
			if (catText.indexOf('\\',searchInd)>0){
				// The main slash is the first
				if (catText.indexOf('/',searchInd)<catText.indexOf('\\',searchInd))
					return catText.indexOf('/',searchInd);
				else 
					return catText.indexOf('\\',searchInd);
			}
			// The first slash must be the /
			return catText.indexOf('/',searchInd);
		}
		// The first slash must be a \
		return catText.indexOf('\\',searchInd);
	}
	
	private Slash createSlash(char slash){
		if (slash=='/') 
			return new Slash(slash,">");
		else 
			return new Slash(slash,"<");
	}
}
