///////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2002-3 Jason Baldridge and University of Edinburgh (Michael White)

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package edinburgh.ccg.ccgCheck.grammar;

import java.util.ArrayList;
import java.util.List;

import edinburgh.ccg.ccgCheck.utils.Arg;
import edinburgh.ccg.ccgCheck.utils.ArgStack;
import edinburgh.ccg.ccgCheck.utils.AtomCat;
import edinburgh.ccg.ccgCheck.utils.BasicArg;
import edinburgh.ccg.ccgCheck.utils.Category;
import edinburgh.ccg.ccgCheck.utils.ComplexCat;
import edinburgh.ccg.ccgCheck.utils.SetArg;
import edinburgh.ccg.ccgCheck.utils.Slash;
import edinburgh.ccg.ccgCheck.utils.TargetCat;
import edinburgh.ccg.ccgCheck.utils.unify.GSubstitution;
import edinburgh.ccg.ccgCheck.utils.unify.GUnifier;
import edinburgh.ccg.ccgCheck.utils.unify.Substitution;
import edinburgh.ccg.ccgCheck.utils.unify.UnifyFailure;

/**
 * Super class for composition rules.
 *
 * @author  Jason Baldridge
 * @author  Michael White
 * @version $Revision: 1.6 $, $Date: 2005/10/18 22:20:15 $
 */
public abstract class AbstractCompositionRule extends AbstractApplicationRule {
	protected boolean _isHarmonic;
	protected Slash _argSlash;

	protected List<Category> apply (Category xyCat, Category yzCat)
	throws UnifyFailure {

		if (xyCat instanceof ComplexCat && yzCat instanceof ComplexCat) {
			List<Category> results = new ArrayList<Category>(1);
			ComplexCat xyCC = (ComplexCat)xyCat;
			ComplexCat yzCC = (ComplexCat)yzCat;

			Arg xyOuter = xyCC.getOuterArg();
			if (xyOuter instanceof BasicArg) {
				Slash xySlash = ((BasicArg)xyOuter).getSlash();
				xySlash.unifyCheck(_functorSlash);
				Category xyOuterCat = ((BasicArg)xyOuter).getCat();

				if (xyOuterCat instanceof AtomCat) {
					// e.g. s/s Y/Z
					ArgStack zStack = yzCC.getArgStack(); 
					zStack.slashesUnify(_argSlash);
					Substitution sub = new GSubstitution();
					GUnifier.unify(xyOuterCat, yzCC.getTarget(), sub);
					xySlash = (Slash)xySlash.fill(sub);
					xySlash.unifyCheck(_functorSlash);
					Category outcome = createResult(xyCC.getResult(), zStack, xySlash, sub);
					results.add(outcome);
				} else if (xyOuterCat instanceof ComplexCat) {
					// e.g. s/(s/n) Y/Z
					Substitution sub = new GSubstitution();
					ArgStack zStack = composeComplexY((ComplexCat)xyOuterCat, xySlash, yzCC, sub);
					xySlash = (Slash)xySlash.fill(sub);
					xySlash.unifyCheck(_functorSlash);
					Category outcome = createResult(xyCC.getResult(), zStack, xySlash, sub); 
					results.add(outcome);
				}
			} else if (xyOuter instanceof SetArg) {
				// e.g. s/{s,n} Y/Z
				Category yzTarget = yzCC.getTarget();
				SetArg xyOuterSet = (SetArg)xyOuter;
				int targetIndex = xyOuterSet.indexOf(yzTarget);
				if (targetIndex > -1) {
					Slash xySlash = xyOuterSet.get(targetIndex).getSlash();
					xySlash.unifyCheck(_functorSlash);
					Substitution sub = new GSubstitution();
					GUnifier.unify(xyOuterSet.getCat(targetIndex), yzTarget, sub);
					Category result = xyCC.copy();
					((ComplexCat)result).setOuterArgument(xyOuterSet.copyWithout(targetIndex));
					ArgStack zStack = yzCC.getArgStack();
					zStack.slashesUnify(_argSlash);
					Category outcome = createResult(result, zStack, xySlash, sub);
					results.add(outcome);
				} else {
					boolean success = false;
					for (int i=0; i<xyOuterSet.size(); i++) {
						BasicArg yInSet = xyOuterSet.get(i);
						if (yInSet.getCat() instanceof ComplexCat) {
							Slash xySlash = yInSet.getSlash();
							xySlash.unifyCheck(_functorSlash);
							ComplexCat yCat = (ComplexCat)yInSet.getCat();
							Substitution sub = new GSubstitution();
							ArgStack zStack = composeComplexY((ComplexCat)yCat, xySlash, yzCC, sub);
							xySlash = (Slash)xySlash.fill(sub);
							xySlash.unifyCheck(_functorSlash);
							Category result = xyCC.copy();
							((ComplexCat)result).setOuterArgument(xyOuterSet.copyWithout(i));
							Category outcome = createResult(result, zStack, xySlash, sub);
							results.add(outcome);
							success = true;
						}
					}
					if (!success) {
						throw new UnifyFailure();
					}
				}
			} else {
				throw new UnifyFailure();
			}

			return results;
		} else {
			throw new UnifyFailure();
		}

	}

	private Category createResult (Category result,	ArgStack zStack, Slash xySlash,	Substitution sub) throws UnifyFailure {
		((GSubstitution)sub).condense();
		result = (Category)result.fill(sub);
		ArgStack newStack = zStack.fill(sub);
		if (result instanceof ComplexCat) {
			((ComplexCat)result).add(newStack);
		} else {
			result = new ComplexCat((TargetCat)result, newStack);
		}
		return result;
	}

	private ArgStack composeComplexY (ComplexCat xyOuterCC, Slash xySlash,
			ComplexCat yzCC, Substitution sub)
	throws UnifyFailure {

		GUnifier.unify(xyOuterCC.getTarget(),
				yzCC.getTarget(),
				sub);
		ArgStack zStack = yzCC.getArgStack();
		if (xyOuterCC.containsDollarArg()) {
			// e.g. s$/(s$\n) s\n/n
			xyOuterCC.getArgStack().unifyPrefix(zStack,
					zStack.size()-1,
					sub);
			zStack = zStack.subList(zStack.size()-1);
			zStack.slashesUnify(_argSlash);
			xySlash = (Slash)xySlash.fill(sub);
			xySlash.unifyCheck(_functorSlash);
			return zStack;
		} else if (xyOuterCC.arity() == 1) {
			ArgStack yzStack = yzCC.getArgStack();
			if (!(xyOuterCC.getArg(0) instanceof BasicArg)) {
				throw new UnifyFailure();
			}
			BasicArg xyOuterOuter = (BasicArg)xyOuterCC.getArg(0);
			Arg yzStackInner = yzStack.get(0);
			if (yzStackInner instanceof SetArg) {
				// e.g. s/(s/n) s/{s,n}
				SetArg yzSetArg = (SetArg)yzStackInner;
				int iaIndex = yzSetArg.indexOf(xyOuterOuter);
				if (iaIndex == -1)
					throw new UnifyFailure();
				xyOuterOuter.unify(yzSetArg.get(iaIndex), sub);
				xySlash = (Slash)xySlash.fill(sub);
				xySlash.unifyCheck(_functorSlash);
				zStack = yzStack.copy();
				zStack.set(0, yzSetArg.copyWithout(iaIndex));
				zStack.slashesUnify(_argSlash);
				return zStack;
			} else {
				// e.g. s/(s/n) s/n/s
				if (yzStack.size() < 2) {
					throw new UnifyFailure();
				}
				xyOuterOuter.unify(yzStackInner, sub);
				zStack = yzStack.subList(1).copy();
				zStack.slashesUnify(_argSlash);
				xySlash = (Slash)xySlash.fill(sub);
				xySlash.unifyCheck(_functorSlash);
				return zStack;
			}
		} else {
			throw new UnifyFailure();
		}
	}
}

