///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2002 Jason Baldridge
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////

package edinburgh.ccg.ccgCheck.grammar;

import java.util.List;

import edinburgh.ccg.ccgCheck.utils.Category;
import edinburgh.ccg.ccgCheck.utils.Slash;
import edinburgh.ccg.ccgCheck.utils.unify.UnifyFailure;

/**
 * Forward composition, e.g. X/Y Y/Z => X/Z
 * 
 * @author Jason Baldridge
 * @version $Revision: 1.2 $, $Date: 2005/10/18 22:20:15 $
 */
public class ForwardComposition extends AbstractCompositionRule {

	public ForwardComposition() {
		this(true);
	}

	public ForwardComposition(boolean isHarmonic) {
		_isHarmonic = isHarmonic;
		if (isHarmonic) {
			_name = ">B";
			_functorSlash = new Slash('/', "^");
			_argSlash = new Slash('/', "^");
		} else {
			_name = ">Bx";
			_functorSlash = new Slash('/', "x");
			_argSlash = new Slash('\\', "x");
		}
		_functorSlash.setAbility("active");
	}

	public List<Category> applyRule(Category[] inputs) throws UnifyFailure {
		if (inputs.length != 2) {
			throw new UnifyFailure();
		}

		return apply(inputs[0], inputs[1]);
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("X").append(_functorSlash.toString()).append("Y Y").append(
				_argSlash.toString()).append("Z => X").append(
				_argSlash.toString()).append("Z");
		return sb.toString();
	}

}
