///////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2003-6 Jason Baldridge, Gann Bierner and 
//Michael White (University of Edinburgh, The Ohio State University)

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package edinburgh.ccg.ccgCheck.grammar;

import java.util.ArrayList;
import java.util.List;

import edinburgh.ccg.ccgCheck.utils.Category;

/**
 * A set of rules for combining categories.
 *
 * @author      Jason Baldridge
 * @author      Gann Bierner
 * @author      Michael White
 * @version     $Revision: 1.20 $, $Date: 2007/06/24 04:13:52 $
 */
public class RuleGroup {
	// rules
	private List<Rule> unaryRules = new ArrayList<Rule>();
	private List<Rule> binaryRules = new ArrayList<Rule>();

	// rule for use in applying coarticulations
	private BackwardApplication bapp = new BackwardApplication();

	/**
	 * Constructs an empty rule group for the given grammar.
	 */
	public RuleGroup() {
		bapp.setRuleGroup(this);
	}

	/** Adds the given rule. */
	public void addRule(Rule r) {
		r.setRuleGroup(this);
		if (r.arity() == 1) { unaryRules.add(r); } 
		else if (r.arity() == 2) { binaryRules.add(r); } 
		else {
			// shouldn't happen
			throw new RuntimeException("Can't determine arity of rule: " + r);
		}
	}

	/** Returns the unary rules. */
	public List<Rule> getUnaryRules() { return unaryRules; }

	/** Returns the binary rules. */
	public List<Rule> getBinaryRules() { return binaryRules; }


	/** Applies the unary rules to the given input sign, returning the list of results. */
	public List<Category> applyUnaryRules(Category input) {
		Category[] inputs = { input };
		List<Category> results = new ArrayList<Category>(2);
		for (Rule r : unaryRules) {
			// apply rule
			((AbstractRule)r).applyRule(inputs, results, "unary");
		}
		return results;
	}

	/** Applies the binary rules to the given input signs, returning the list of results. */
	public List<Category> applyBinaryRules(Category input1, Category input2) {
		Category[] inputs = { input1, input2 };
		List<Category> results = new ArrayList<Category>(2);
		for (Rule r : binaryRules) {
			String expansion;
			if (r.name().contains(">"))	expansion="headLeft";
			else expansion="headRight";
			// apply rule
			((AbstractRule)r).applyRule(inputs, results, expansion);
		}
		return results;
	}
}
