///////////////////////////////////////////////////////////////////////////////
//// Copyright (C) 2003-4 Gunes Erkan and University of Edinburgh (Michael White)
//// 
//// This library is free software; you can redistribute it and/or
//// modify it under the terms of the GNU Lesser General Public
//// License as published by the Free Software Foundation; either
//// version 2.1 of the License, or (at your option) any later version.
//// 
//// This library is distributed in the hope that it will be useful,
//// but WITHOUT ANY WARRANTY; without even the implied warranty of
//// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//// GNU Lesser General Public License for more details.
//// 
//// You should have received a copy of the GNU Lesser General Public
//// License along with this program; if not, write to the Free Software
//// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

package edinburgh.ccg.ccgCheck.grammar;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;

import edinburgh.ccg.ccgCheck.utils.unify.SimpleType;

/**
 * Class for constructing and holding the hierarchical simple type maps.
 *
 * @author  Gunes Erkan
 * @author  Michael White
 * @version $Revision: 1.11 $, $Date: 2006/11/17 04:46:07 $
 */
public class Types {

    private final HashMap<String,SimpleType> nameToType = new HashMap<String,SimpleType>();
    private final ArrayList<SimpleType> indexToType = new ArrayList<SimpleType>();
    private int maxTypeIndex = 0;
    public static final String TOP_TYPE = "top";
    public static final String BOT_TYPE = "bottom";
	
    /** Constructor for an empty hierarchy (with just the top type). */
    public Types() {
        getSimpleType(TOP_TYPE);
    }

    /** Returns the simple type with the given name, or a new one if none yet exists. */
    public SimpleType getSimpleType(String typeName) {
        SimpleType type = nameToType.get(typeName);
        if (type == null) {
            BitSet bs = new BitSet();
            bs.set(maxTypeIndex);
            SimpleType newtype = new SimpleType(maxTypeIndex, typeName, bs, this);
            nameToType.put(typeName, newtype);
            indexToType.add(newtype);
            nameToType.get(TOP_TYPE).getBitSet().set(maxTypeIndex++);
            return newtype;
        }
        else return type;
    }

    /** Returns whether there is a simple type with the given name. */
    public boolean containsSimpleType(String typeName) {
        return nameToType.containsKey(typeName);
    }
    
    /** Returns the list of types, with parents preceding children in the hierarchy. */
    public ArrayList<SimpleType> getIndexMap() {
        return indexToType;
    }

    /**
     * Prints the types and their subtypes to System.out.
     */
    public void printTypes() {
        System.out.println("types:");
        for (int i=0; i < indexToType.size(); i++) {
            SimpleType st = indexToType.get(i); 
            System.out.println(i + ": " + st.getName() + " subtypes: " + st.getBitSet());
        }
        System.out.println();
    }
}
