///////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2003-5 Jason Baldridge, Gann Bierner and 
//University of Edinburgh (Michael White)

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package edinburgh.ccg.ccgCheck.utils;

import edinburgh.ccg.ccgCheck.utils.unify.Substitution;
import edinburgh.ccg.ccgCheck.utils.unify.UnifyFailure;
import edinburgh.ccg.ccgCheck.utils.unify.Variable;

/**
 * The most basic CG category.  This corresponds to a category like 'np[acc]',
 * i.e. a category name and associated features.
 *
 * @author      Gann Bierner
 * @author      Jason Baldridge
 * @author      Michael White
 * @version     $Revision: 1.13 $, $Date: 2006/10/12 21:20:48 $
 */
public final class AtomCat extends AbstractCat implements TargetCat {

	private static final long serialVersionUID = 1L;

	private String type;

	/** Constructor which creates an atomic category with the given type. */
	public AtomCat(String t) {
		type = t;
	}

	/**
	 * Returns this category as the target category.
	 */
	public TargetCat getTarget() { return this; }


	public String getType() {
		return type;
	}


	public Category copy() {
		return new AtomCat(type);
	}

	public void unifyCheck (Object u) throws UnifyFailure {
		if (u instanceof AtomCat) {
			AtomCat u_ac = (AtomCat)u;

			//(Chris) This should check the function types */
			// if we are dealing with an xyOuterCat, then if it doesn't
			// have any specific function type it should be allowed to unify
			// otherwise, it should be allowed to unify only with a category
			// of the same function type
			// this is the xyOuterCat
			// u_ac is the yCat

			// Orginal
			//if (!(type.equals(u_ac.type))) {
			//	throw new UnifyFailure();
			//}
			if (!(typeEqual(type,u_ac.type))){
				throw new UnifyFailure();
			}
		} else if (!(u instanceof Variable)) {
			throw new UnifyFailure();
		}
	}

	private boolean typeEqual (String restrictedType, String yType){
		//(Chris) If restrictedType has a function
		if (restrictedType.indexOf('[')>0){
			// If the functions (and the category types) are the same
			if (yType.indexOf('[')>0){
				if (restrictedType.substring(restrictedType.indexOf('['), restrictedType.indexOf(']')).
						equals(yType.substring(yType.indexOf('['), yType.indexOf(']')))
						&&
						restrictedType.substring(0, restrictedType.indexOf('[')).
						equals(yType.substring(0, yType.indexOf('['))))
					return true;
			}
		}
		//(Chris) Else if the category types are the same (no matter the yType's function) 
		else {
			if (yType.indexOf('[')>0){
				if (restrictedType.equals(yType.substring(0, yType.indexOf('['))))
					return true;
			}
			else {
				if (restrictedType.equals(yType))
					return true;
			}
		}
		return false;
	}

	/** NB: The LF does not participate in unification. */
	public Object unify (Object u, Substitution sub) 
	throws UnifyFailure {

		//(Chris) Again here this should also check function types*/
		// Original
		//if (u instanceof AtomCat && type.equals(((AtomCat)u).type))
		if (u instanceof AtomCat && typeEqual(type,((AtomCat)u).type)) {
			return new AtomCat(type);
		}
		else {
			throw new UnifyFailure();
		}
	}

	public Object fill (Substitution s) throws UnifyFailure {
		AtomCat $ac = new AtomCat(type);
		return $ac;
	}

	public boolean shallowEquals(Object c) {
		if (c instanceof AtomCat) {
			AtomCat ac = (AtomCat)c;
			return type.equals(ac.type);
		}
		return false;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(type);
		if (sb.length() == 0) return "UnknownCat";
		return sb.toString();
	}

	/**
	 * Returns the supertag for the category.
	 */
	public String getSupertag() {
		StringBuffer sb = new StringBuffer();
		sb.append(type);
		if (sb.length() == 0) return "UnknownCat";
		return sb.toString();
	}

	public String toTeX() {
		StringBuffer sb = new StringBuffer();
		sb.append(type);
		if (sb.length() == 0) return "UnknownCat";
		return sb.toString();
	}
}
