///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2002-5 Jason Baldridge, Gann Bierner and 
//                      University of Edinburgh (Michael White)
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////

package edinburgh.ccg.ccgCheck.utils;

import edinburgh.ccg.ccgCheck.utils.unify.Unifiable;

/**
 * A CCG category.
 *
 * @author      Jason Baldridge
 * @author      Gann Bierner
 * @author      Michael White
 * @version     $Revision: 1.9 $, $Date: 2005/10/18 22:20:15 $
 */
public interface Category extends Unifiable, java.io.Serializable {

	/**
	 * Determines if this category is equal to another on the top level.
	 * It does not check sub categories.
	 *
	 * @param o object to check for equality
	 * @return whether or not this is shallowly equal to object
	 */
	public boolean shallowEquals(Object o);

	/**
	 * Deep copies this category.
	 *
	 * @return a deep copy of this category
	 */
	public Category copy();

	/**
	 * Iterates through this Category applying a function to this category
	 * and every subcategory.
	 *
	 * @param f a function to be applied
	 */    
	public void forall(CategoryFcn f); //to ls


	/** 
	 * Returns a hash code for this category. 
	 * The hash code handles equivalence up to variable names 
	 * as long as features and predicates are in the same order.
	 */
	public int hashCode();

	/** 
	 * Returns whether this category equals the given object. 
	 * Equivalence up to variable names is handled  
	 * as long as features and predicates are in the same order.
	 */
	public boolean equals(Object obj);


	/**
	 * Returns the target category of this category.
	 */
	public TargetCat getTarget();

	/**
	 * Returns the supertag for the category.
	 */
	public String getSupertag();

	/**
	 * Returns a TeX-formatted string representation for the category.
	 */
	public String toTeX();
}
