//////////////////////////////////////////////////////////////////////////////
//Copyright (C) 2011-12 Christos Christodoulopoulos and Tejaswini Deoskar

//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 3 of the License, or (at your option) any later version.

//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package edinburgh.ccg.probModel;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.HashMap;

import edinburgh.ccg.probModel.util.BinaryLeaf;
import edinburgh.ccg.probModel.util.BinaryNode;
import edinburgh.ccg.probModel.util.BinaryTree;
import edinburgh.ccg.probModel.util.SmoothLexCreator;
import edinburgh.ccg.util.Logger;

public class ProbModelBaseTrainer extends ProbModelTrainer {
	
	public ProbModelBaseTrainer(Logger log){
		super(log);
	}
	
	/**
	 * This method includes the test words and smoothes lexical frequencies
	 * before computing the probability model.
	 */
	protected void trainModel(BufferedReader trainBuffer, BufferedReader testBuffer) throws IOException{
		System.out.println("Training model for base");
		String line = null;
		BinaryNode tree;
		SmoothLexCreator smoothLex = new SmoothLexCreator(log) ;
		line = trainBuffer.readLine();
		// Get the counts
		while ( line != null && (line.indexOf("###") != -1 || line.indexOf("ID=") != -1)){
			line = trainBuffer.readLine();
			tree = new BinaryNode(); 
			BinaryTree.addEntireTree(line.substring(line.indexOf("(")));
			
			if (tree.head != null){					
					updateCounts(tree);
			}
			tree = null;
											
			line = trainBuffer.readLine();
		}
		//Remove rare categories
		if (o.getCatThreshold() > 0) removeRareCats();
		
		//IF SMOOTH TRAINING IS ON
		if (testBuffer != null){
			//Read in the POS-tagged test data.
			line = "";
			while ( line != null) {
				line = testBuffer.readLine();
				if (line != null) {
					smoothLex.updateTestCounts(line);
				}
			}
			//Smooth the Lexicon and add test words
			smoothLex.smooth();
			//Write out a frequency lexicon. Used for merging with expected counts during semi-supervised EM. 
			smoothLex.printSmoothedLex(o.getModelsDir(), "wordFreqModel");
			// Now compute the probabilities
			computeModel(smoothLex.smoothedWordCatFreq);
		}
		else {
			//Replace rare words with their tags
			if (o.useTags()) replaceRareWords();
			//Now compute the probabilities
			computeModel(null);
			computeModel(null);
		}
		
		// Store the new models
		writeModels();
		
	}
		
	// Original updatecounts code, before lexicalised version of Chris
	protected void updateCounts(BinaryTree node){
		incrementFreq(expNum, node.expCat());
		incrementFreq(expDenom, node.cat());	
		if (node instanceof BinaryNode)
			updateCounts((BinaryNode)node);
		if (node instanceof BinaryLeaf)
			updateCounts((BinaryLeaf)node);	
	}
	
	protected void updateCounts(BinaryNode node){
		if (node.cat.equals("TOP")){
			incrementFreq(expNum, node.expCat());
			incrementFreq(expDenom, node.cat());		 
		}
		incrementFreq(headNum, node.headExpCat());
		if ((node.exp == BinaryTree.leftHeadExp || node.exp == BinaryTree.rightHeadExp) && node.sister != null){
			incrementFreq(sisterNum, node.sisterHeadExpCat());	    
			updateCounts(node.sister);	  
		}	
		updateCounts(node.head);
	}
	
	protected void computeModel(HashMap<String, Double> smoothWordCatCounter){
		// Compute the wordProb: P(w|cat)=count(word,cat)/count(cat as leaf)
		//								 =wordCatCounter/catLeafCounter
		if (smoothWordCatCounter != null) 
			computeProbs(smoothWordCatCounter,wordDenom,wordProb);
		else computeProbs(wordNum,wordDenom,wordProb);

		// Compute the expProb: P(exp|cat)=count(exp,cat)/count(cat)
		// 								  =expCatCounter/catCounter
		computeProbs(expNum,expDenom,expProb);

		// Compute the headCatProb: P(H|exp,cat)=count(H,exp,cat)/count(exp,cat)
		// 										=headExpCatCounter/expCatCounter
		computeProbs(headNum,expNum,headProb);

		// Compute the sisterCatProb: P(S|H,exp,cat)=count(S,H,exp,cat)/count(H,exp,cat)
		// 											=sisterHeadExpCatCounter/headExpCatCounter
		computeProbs(sisterNum,headNum,sisterProb);
		// Clear the frequency counters and collect garbage
		wordNum = null;
		wordDenom = null;
		expNum = null;
		expDenom = null;
		headNum = null;
		sisterNum = null;

		System.gc();
	}
}
