package edinburgh.ccg.probModel;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.HashMap;

import edinburgh.ccg.probModel.util.BinaryLeaf;
import edinburgh.ccg.probModel.util.BinaryNode;
import edinburgh.ccg.probModel.util.BinaryTree;
import edinburgh.ccg.probModel.util.SmoothLexCreator;
import edinburgh.ccg.util.Logger;

public class ProbModelLexCatTrainer extends ProbModelTrainer{

	
	//Constructor
	public ProbModelLexCatTrainer(Logger log){
		super(log);
	}
	
	/**
	 * This method is almost the same as the super class, but calls local methods.
	 * Might have to call lexicalised tree, for model types that require access to the word as well 
	 * as the lexical category. 
	 * Could possibly be moved to the superclass ProbModelTrainer in order to avoid code duplication
	 */
	protected void trainModel(BufferedReader trainBuffer, BufferedReader testBuffer) throws IOException{
		System.out.println("Training model for LexCat");
		
		//Extra data structures for Lex Cat model base model
		// probability maps
		//expansion model
		backOffExpProb = new HashMap<String,Double>();
		newExpProb = new HashMap<String,Double>(); 
		newBackOffExpProb = new HashMap<String,Double>();
		
		//headCat model
		backOffHeadProb = new HashMap<String,Double>();
		newHeadProb = new HashMap<String,Double>();
		newBackOffHeadProb = new HashMap<String,Double>(); 
		
		//sistercat model
		backOffSisterProb = new HashMap<String,Double>();
		newSisterProb = new HashMap <String, Double>();
		newBackOffSisterProb = new HashMap <String, Double>();
		
		//sis LexCat model
		sisterLexCatProb = new HashMap<String, Double>();
		backOff1SisterProb = new HashMap<String, Double>();
		newSisterLexCatProb = new HashMap<String, Double>();
		newBackOffSisterLexCatProb = new HashMap<String, Double>();
		
		//Extra counters for LexCat Model
		backOffExpNum = new HashMap<String,Integer>();
		backOffExpDenom = new HashMap<String,Integer>();
		
		backOffHeadNum = new HashMap<String,Integer>();
		backOffSisterNum = new HashMap<String,Integer>();
		
		
		sisterLexCatNum = new HashMap<String,Integer>();
		backOffSisterLexCatNum = new HashMap<String,Integer>();
		backOffSisterLexCatDenom = new HashMap<String,Integer>();
		
		String line = null;
		BinaryNode tree;
		SmoothLexCreator smoothLex = new SmoothLexCreator(log) ;
		line = trainBuffer.readLine();
		// Get the counts
		while ( line != null && (line.indexOf("###") != -1 || line.indexOf("ID=") != -1)){
			line = trainBuffer.readLine();
			tree = new BinaryNode(); 
			BinaryTree.addEntireTree(line.substring(line.indexOf("(")));
			
			if (tree.head != null){
				updateCounts(tree);								
			}
			tree = null;
											
			line = trainBuffer.readLine();
		}
		//Remove rare categories
		if (o.getCatThreshold() > 0) removeRareCats();
		
		//IF SMOOTH TRAINING IS ON
		if (testBuffer != null){
			//Read in the POS-tagged test data.
			line = "";
			while ( line != null) {
				line = testBuffer.readLine();
				if (line != null) {
					smoothLex.updateTestCounts(line);
				}
			}
			//Smooth the Lexicon and add test words
			smoothLex.smooth();
			//Write out a frequency lexicon. Used for merging with expected counts during semi-supervised EM. 
			smoothLex.printSmoothedLex(o.getModelsDir(), "wordFreqModel");
			// Now compute the probabilities
			computeModel(smoothLex.smoothedWordCatFreq);
		}
		else {
			//Replace rare words with their tags
			if (o.useTags()) replaceRareWords();
			//Now compute the probabilities
			computeModel(null);
			computeModel(null);
		}
		// Store the new models
		 writeModels();
	}
	
protected void computeModel(HashMap<String, Double> smoothWordCatCounter){
		// Compute the wordProb: P(w|cat)=count(word,cat)/count(cat as leaf)
		//								 =wordCatCounter/catLeafCounter
		if (smoothWordCatCounter != null) 
			computeProbs(smoothWordCatCounter,wordDenom,wordProb);
		else computeProbs(wordNum,wordDenom,wordProb);

		// Compute the expProb: P(exp|cat, lexcat)=count(exp, cat, lexcat)/count(cat, lexcat )
		// 								  =expCatCounter/catCounter
		computeProbs(expNum,expDenom, expProb);
		//Backoff for the expProb
		
		computeProbs(backOffExpNum, backOffExpDenom, backOffExpProb);
		computeBackOff(newExpProb,
					   newBackOffExpProb,
					   expProb,
					   backOffExpProb,
					   expNum,
					   expDenom);
		//Replace  with the backed off probabilities. 
		expProb.clear();
		expProb.putAll(newExpProb); //This contains the backed off probabilities now. 
		backOffExpProb.clear();
		backOffExpProb.putAll(newBackOffExpProb);
		
		// Compute the headCatProb: P(H|exp,cat, lexcat)=count(H,exp,cat, lexcat)/count(exp,cat, lexcat)
		// 										=headExpCatCounter/expCatCounter
		computeProbs(headNum,expNum,headProb);
		//BackOff for the HeadCatProb
		
		computeProbs(backOffHeadNum, backOffExpNum, backOffHeadProb);
		computeBackOff(newHeadProb,
					   newBackOffHeadProb,
					   headProb,
					   backOffHeadProb,
					   headNum,
					   expNum);
		//Replace  with the backed off probabilities. 
		headProb.clear();
		headProb.putAll(newHeadProb); //This contains the backed off probabilities now. 
		backOffHeadProb.clear();
		backOffHeadProb.putAll(newBackOffHeadProb);
		
		// Compute the sisterCatProb: P(S|H,exp,cat, lexcat)=count(S,H,exp,cat, lexcat)/count(H,exp,cat, lexcat)
		// 											=sisterHeadExpCatCounter/headExpCatCounter
		computeProbs(sisterNum,headNum,sisterProb);
		
		// Compute the backoff for sisterCatProb: P(S|H,exp,cat)=count(S,H,exp,cat)/count(H,exp,cat)
		// 											=backOffSisterHeadExpCatCounter/backOffHeadExpCatCounter
		computeProbs(backOffSisterNum, backOffHeadNum, backOffSisterProb);
		
		// Back off 
		computeBackOff(newSisterProb, 
					   newBackOffSisterProb,
				       sisterProb, 
				       backOffSisterProb, 
				       sisterNum, 
				       headNum);
		
		//Replace sisCatProbabilities with the backed off probabilities. 
		sisterProb.clear();
		sisterProb.putAll(newSisterProb); //This contains the backed off probabilities now. 
		backOffSisterProb.clear();
		backOffSisterProb.putAll(newBackOffSisterProb);
		
		//For the sister LexCat model
		//Compute P(Cs|S~H,exp,P)
		computeProbs(sisterLexCatNum, backOffSisterNum, sisterLexCatProb) ;
		computeProbs(backOffSisterLexCatNum,backOffSisterLexCatDenom,backOff1SisterProb );
		computeBackOff(newSisterLexCatProb, //new backed-off final probs
				       newBackOffSisterLexCatProb,
				       sisterLexCatProb,  //fine-grained probs ( to be backed off)
				       backOff1SisterProb, //coarser probs (to back off to)
				       sisterLexCatNum, //numMap
				       backOffSisterNum);
		//Replace sisLexCatProbabilities with the backed off probabilities. 
		sisterLexCatProb.clear();
		sisterLexCatProb.putAll(newSisterLexCatProb); //This contains the backed off probabilities now. 
		backOff1SisterProb.clear();
		backOff1SisterProb.putAll(newBackOffSisterLexCatProb);
		
		// Clear the frequency counters and collect garbage
		wordNum = null;
		wordDenom = null;
		expNum = null;
		expDenom = null;
		headNum = null;
		sisterNum = null;
		sisterLexCatNum= null;
		System.gc();
	}
	
	//For model type LexCat
	protected void updateCounts(BinaryTree node){
		String expCatLexCat = node.expCat()+"||"+node.lexCat();		
		String catLexCat = node.cat()+"||"+node.lexCat();
		incrementFreq(expNum,expCatLexCat);
		incrementFreq(expDenom, catLexCat);	
		//backoffs
		incrementFreq(backOffExpNum,node.expCat());
		incrementFreq(backOffExpDenom,node.cat());
		
		if (node instanceof BinaryNode)
			updateCounts((BinaryNode)node);
		if (node instanceof BinaryLeaf)
			updateCounts((BinaryLeaf)node);	
	}
	
	//For model type LexCat
	protected void updateCounts(BinaryNode node){
		String expCatLexCat = node.expCat()+"||"+node.lexCat();			
		String catLexCat = node.cat()+"||"+node.lexCat();
		if (node.cat.equals("TOP")){
			incrementFreq(expNum,expCatLexCat);
			incrementFreq(expDenom, catLexCat);
			//backoffs
			incrementFreq(backOffExpNum,node.expCat()); //without lexcat
			incrementFreq(backOffExpDenom,node.cat());
		}
		
		String headExpCatLexCat = node.headExpCat() +"||"+node.lexCat();
		incrementFreq(headNum,headExpCatLexCat );
		incrementFreq(backOffHeadNum,node.headExpCat() ); //without lexcat
		
		if ((node.exp == BinaryTree.leftHeadExp || node.exp == BinaryTree.rightHeadExp) && node.sister != null){
			String sisterHeadExpCatLexCat = node.sisterHeadExpCat() +"||"+node.lexCat();
			incrementFreq(sisterNum, sisterHeadExpCatLexCat);
			
			incrementFreq(backOffSisterNum, node.sisterHeadExpCat());
			updateCounts(node.sister);	 
			
			//Counters for lexcat of sister (non-head)
			String sisterLexCat = node.sister.lexCat(); //lexical category of the sister
			
			//S|H|exp|Cat||sisterLexCat : sisterLexCat is appended to the end to keep things
			//uniform with other counter, although the variable name has sisterLexCat at the beginning
			String sisterLexCatSisterHeadExpCat = sisterLexCat +"|" + node.sisterHeadExpCat();
			incrementFreq(sisterLexCatNum, sisterLexCatSisterHeadExpCat );
			
			String sisterLexCatHeadExpCat = sisterLexCat +"|" +node.sisterCat();
			incrementFreq(backOffSisterLexCatNum, sisterLexCatHeadExpCat);
			incrementFreq(backOffSisterLexCatDenom, node.sisterCat());
		}	
		updateCounts(node.head);
	}
	
	protected void writeModels(){
		// Store the new models
		writeModel(o.getModelsDir(),o.getWordProbModelFile(),wordProb, false);
		
		writeModel(o.getModelsDir(),o.getExpProbModelFile(),expProb, false);
		writeModel(o.getModelsDir(),o.getExpProbModelFile(),backOffExpProb, true);
		
		writeModel(o.getModelsDir(),o.getHeadCatProbModelFile(),headProb, false);
		writeModel(o.getModelsDir(),o.getHeadCatProbModelFile(),backOffHeadProb, true);
		
		writeModel(o.getModelsDir(),o.getSisterCatProbModelFile(),sisterProb, false);	
		writeModel(o.getModelsDir(),o.getSisterCatProbModelFile(),backOffSisterProb, true);
		
		writeModel(o.getModelsDir(),o.getSisterLexCatProbModelFile(),sisterLexCatProb, false);
		writeModel(o.getModelsDir(),o.getSisterLexCatProbModelFile(),backOff1SisterProb, true);
	}
}
