package edinburgh.ccg.probModel.util;

/** The class for leaf nodes.<br>
    Differs from {@link BinaryNode <tt>BinaryNode</tt>} in that it does not have <tt>head, sister or daughter</tt>.<br>
    The counts for the lexical categories are updated only on the leaf nodes.
    During parsing, the lexical heads <tt>headWords</tt> are determined here.
 */
public class BinaryLeaf extends BinaryTree{

	/** the word. */
	private String word;

	/** and its lexical category */
	private String lexcat;

	/** The POS-tag of the word */
	private String tag;
	
	private int index;

	//===================================================
	// THE CONSTRUCTOR
	//===================================================


	/**  The constructor for training. 
     Creates a <tt>HeadWordlist</tt> 
     -- this is necessary for the lexcat model.
     Like {@link BinaryLeaf#BinaryLeaf(int, String, String, String) BinaryLeaf}, the constructor for the parser, except the span of a leaf created by this constructor is 0-0, and this leaf has no probability.

	 * @see HeadWordList#HeadWordList(String, String)
	 */ 
	public  BinaryLeaf(String c, String w, String t){
		super(c);

		this.lexcat = c;
		this.word = w;
		this.tag = t;
		exp = leafExp;
	}
	
	public  BinaryLeaf(String c, String w, String t, int i){
		super(c);
		this.lexcat = c;
		this.word = w;
		this.tag = t;
		exp = leafExp;
		this.index = i;
	}
	
	public BinaryLeaf(BinaryLeaf node){
		super(node.cat);
		this.word = node.word;
		this.tag = node.tag;
		this.lexcat = node.lexcat;
		this.exp = node.exp;
		this.headpos = node.headpos;
		this.parent = null;

		this.head = node.head;
		this.sister = node.sister;
	}



	// METHODS
	// =======  

	//==== headWord()
	/** returns  <tt>word</tt>.
	 * @see BinaryTree#headWords
	 */
	public String headWord(){
		return word;
	}
	
	protected int headIndex(){
		return index;
	}
	
	protected String headCat(){
		return cat;
	}

	//==== lexCat()
	/** returns <tt>cat</tt>
	 * @see BinaryTree#headWords 
	 */
	public String lexCat(){
		return cat;    
	}    

	/** <pre>word|lexcat</pre> */
	public String lexNumKey(){
		return (new StringBuffer(word).append('|').append(lexcat)).toString();
	}
	/** just the lexical category */
	public String lexDenomKey(){
		return lexcat;
	}

	public String tag(){
		return tag;
	}

	public BinaryTree copy(){
		//System.out.println("copy leaves: " + this.headWord()); 
		return new BinaryLeaf(this);// to be implemented	
	}
	
	public BinaryTree deepCopy(){
		return new BinaryLeaf(this);
	}

	@Override
	public String toString() {
		return "(<L "+cat+" "+tag+" "+tag+" "+word+" "+cat+">)";
	}
}
