package edinburgh.ccg.probModel.util;

/** 
 * The class BinaryNode is for internal, non-leaf nodes.
 */
public class BinaryNode extends BinaryTree{


	//***************************************
	// CONSTANTS
	//***************************************

	/** String constant for the root node */
	static final String TOP = new String("TOP");

	//***************************************
	// CONSTRUCTORS
	//***************************************

	/** A dummy constructor.  */
	public BinaryNode(){
		super(TOP);
		current = this;
		exp = unaryExp;
		cat = TOP;	
	}
//	copies individual nodes
	public BinaryNode(BinaryNode node){
		super(node.cat);
		this.exp = node.exp;
		this.headpos = node.headpos;
		this.parent = null;

		this.head = node.head;
		this.sister = node.sister;

	}
	/** The constructor for training. Sets <tt>exp</tt> to <i>headDir</i>. 
     @see BinaryTree#BinaryTree(String)	
	 */ 
	public BinaryNode(String category, String expansion){	
		super(category);		
		exp = expansion;
		if (expansion.equals(leftHeadExp) || expansion.equals(unaryExp)) headpos = 0;
		else headpos = 1;
	}


	//***************************************
	// METHODS
	//***************************************

	public BinaryTree copy(){
		BinaryNode copy = new BinaryNode(this);
		return copy;
	}
	
	public BinaryTree deepCopy(){
		BinaryNode copy = new BinaryNode(this);
		copy.cat = new String(this.cat);
		copy.head = this.head.deepCopy();
		if (copy.sister != null) copy.sister = this.sister.deepCopy();
		return copy;
	}

	public boolean expands(String pCat, String hCat){
		if (head != null && sister == null
				&& this.cat().equals(pCat)
				&& this.head.cat().equals(hCat))
			return true;
		else return false;

	}
	
	/** <pre>exp|cat</pre> */
	public String expCat(){
		return (new StringBuffer(exp).append('|').append(cat)).toString();
	}
	
	/** <pre>H|exp|cat</pre> */
	public String headExpCat(){
		return (new StringBuffer(head.cat()).append('|').append(expCat())).toString();
	}
	
	/** <pre>S|H|exp|cat</pre> */
	public String sisterHeadExpCat(){
		return (new StringBuffer(sister.cat()).append('|').append(headExpCat())).toString();
	}
	
	// sisterCat()
	/** returns the category of sister, 
	or <tt>null</tt> if sister is <tt>null</tt>. */
	public String sisterCat(){   
		if (sister != null)
			return sister.cat();
		else return null;
	}

	//====  sisterLexCat()
	/** returns the lexical category of sister, 
	or <tt>null</tt> if sister is  <tt>null</tt>. */
	public String sisterLexCat(){
		if (sister != null)
			return sister.lexCat();
		else return null;
	}

	/** returns a (shallow) copy of the tree. */
	public BinaryNode copyTree(){
		BinaryNode newTree = null;
		try { newTree = (BinaryNode)this.clone();}
		catch (Exception E) { E.printStackTrace();}
		return newTree;
	}

	public void setHead(BinaryTree tree){
		this.head = tree;
		this.head.parent = this;
	}
	public void setSister(BinaryTree tree){
		if (this.exp != unaryExp){
			this.sister = tree;
			this.sister.parent = this;
		}
	}
	
	protected int headIndex() {
		return head.headIndex();
	}
	
	protected String headWord() {
		return head.headWord();
	}
	
	protected String headCat(){
		return head.headCat();
	}

	public String toString(){
		//(<T cat headPos dtrs_nr> (<head>) (<sister>))
		int dtrs_nr = 1;
		if (sister != null) dtrs_nr = 2; 
		String str = "(";
		str += "<T "+cat+" "+headpos+" "+dtrs_nr+">";
		if (exp.equals(unaryExp)) str += " "+head.toString();
		else {
			//Head first
			if (exp.equals(leftHeadExp) || exp.equals("conj")) {
				str += " "+head.toString();
				str += " "+sister.toString();
			}
			//Sister first
			else {
				str += " "+sister.toString();
				str += " "+head.toString();
			}
		}
		
		str += " )";
		return str;
	}
}

